/*******************************************************************/
/*                                                                 */
/* Module           : matrix.cc                                    */
/* Description      : Implementation file for class Matrix.        */
/* Author           : Sebastian Boenisch                           */
/* Email            : Sebastian.Boenisch@iwr.uni-heidelberg.de     */
/* Copyright        : University of Heidelberg                     */
/* Created on       : (see RCS entry below)                        */
/* $Id: matrix.cc,v 1.7 2002/03/05 17:45:21 boenisch Exp $         */
/* Last modified by :                                              */
/* Last modified on :                                              */
/* Update count     :                                              */
/*                                                                 */
/*******************************************************************/
/*                                                                 */
/*  Date      Name               Changes/Extensions                */
/*  ====      ====               ==================                */
/*                                                                 */
/*******************************************************************/



template<class T>
Matrix<T>::Matrix(int n)
  : vector<vector<T> >(n,vector<T>(n)), nRows(n), nCols(n) {
} // Matrix<T>::Matrix(int n)


template<class T>
Matrix<T>::Matrix(int n, int m)
  : vector<vector<T> >(n,vector<T>(m)), nRows(n), nCols(m) {
} // Matrix<T>::Matrix(int n, int m)


template<class T>
Matrix<T> Matrix<T>::transpose() {
  Matrix<T> result(nCols,nRows);
  for (int i=0; i<nRows; ++i)
    for (int j=0; j<nCols; ++j)
      result[j][i] = (*this)[i][j];

  return result;
}


template<class T>
pair<int,int> Matrix<T>::getSize() const {
  return pair<int,int>(nRows,nCols);
} // pair<int,int> Matrix<T>::getSize()


template<class T>
void Matrix<T>::print(ostream& out = cout) const {
  for (int i=0; i<nRows; i++) {
    for (int j=0; j<nCols; j++) {
      out.width(2);
      out << (*this)[i][j] << " ";
    }
    out << endl;
  }
  out << endl;
} // void Matrix<T>::print()


template<class T>
void Matrix<T>::printMatlabExp(ostream& out = cout) const {
  out << "[" ;
  for (int i=0; i<nRows; i++) {
    for (int j=0; j<nCols; j++) {
      out.width(2);
      out << (*this)[i][j];
      if (j!=nCols-1)
	out << "," ;
    }
    if (i!=nRows-1)
      out << ";";
  }
  out << "]" << endl;
} // void Matrix<T>::printMatlabExp()

template<class T>
bool Matrix<T>::isZero() const {
  bool zero = true;

  for (int i=0; ( (i<nRows) && zero); ++i) {
    for (int j=0; ( (j<nCols) && zero); ++j) {
      if ( (*this)[i][j] != (T)0 )  zero = false;
    }
  }
  
  return zero;
} // bool Matrix<T>::isZero()


template<class T>
bool Matrix<T>::isQuadratic() const {
  return (nRows==nCols);
} // bool Matrix<T>::isQuadratic()



/* ************************************************** */
/*               Non-member functions                 */ 
/* ************************************************** */

/// Prints a vector.
template<class T>
void print(vector<T> b) {
  for (int i=0; i<b.size(); i++)
    cout << b[i] << " ";
  cout << endl;
} // void print(vector<T> b)


/// Input operator for matrices.
template <class T>
istream& operator>>(istream& inStream, Matrix<T>& matrix) {
  for (int i=0; i<matrix.getSize().first; i++)
    for (int j=0; j<matrix.getSize().second; j++)
      inStream >> matrix[i][j];    
  
  return inStream;
} // istream& operator>>(...
 
