/*******************************************************************/
/*                                                                 */
/* Module           : matrix.hh                                    */
/* Description      : Header file for class Matrix, a very simple  */
/*                    class for matrices.                          */
/* Author           : Sebastian Boenisch                           */
/* Email            : Sebastian.Boenisch@iwr.uni-heidelberg.de     */
/* Copyright        : University of Heidelberg                     */
/* Created on       : (see RCS entry below)                        */
/* $Id: matrix.hh,v 1.10 2002/03/05 17:45:30 boenisch Exp $        */
/* Last modified by :                                              */
/* Last modified on :                                              */
/* Update count     :                                              */
/*                                                                 */
/*******************************************************************/
/*                                                                 */
/*  Date      Name               Changes/Extensions                */
/*  ====      ====               ==================                */
/*                                                                 */
/*******************************************************************/


#ifndef MATRIX_HH
#define MATRIX_HH


#include <iostream>
#include <utility>
#include <vector>


/// A very simple matrix class.
template <class T>
class Matrix : public vector<vector<T> > {

private:

  /* ************************************************** */
  /*                      Variables                     */ 
  /* ************************************************** */
  
  /// Number of rows.
  int nRows;

  /// Number of columns.
  int nCols;


public:

  /* ************************************************** */
  /*                 Methods (public)                   */ 
  /* ************************************************** */ 

  // Constructors

  /// Constructs a quadratic nxn matrix.
  Matrix(int n);
  /// Constructs a matrix with n rows and m columns.
  Matrix(int n, int m);

  /// Returns the pair <number of rows,number of columns>.
  pair<int,int> getSize() const;

  /// Returns the matrix with exchanged dimensions.
  Matrix<T> transpose();

  /// Prints the matrix.
  void print(ostream& = cout) const;

  /// Prints the matrix in Matlab format.
  void printMatlabExp(ostream& = cout) const;

  /// Returns true iff *this is the zero matrix.
  bool isZero() const;

  /// Returns true iff n=m.
  bool isQuadratic() const;

}; // class Matrix 



/* ************************************************** */
/*               Non-member functions                 */ 
/* ************************************************** */

/// Prints a vector.
template<class T>
void print(vector<T> b);

/// Overloads the input operator for matrices.
template <class T>
istream& operator>>(istream& inStream, Matrix<T>& matrix);



/* ************************************************** */
/*             Include implementation                 */ 
/* ************************************************** */

#include "matrix.cc"

#endif 
