/*******************************************************************/
/*                                                                 */
/* Module           : rational.hh                                  */
/* Description      : Header file for class Rational, a class for  */
/*                    rational numbers.                            */
/* Author           : Sebastian Boenisch                           */
/* Email            : Sebastian.Boenisch@iwr.uni-heidelberg.de     */
/* Copyright        : University of Heidelberg                     */
/* Created on       : (see RCS entry below)                        */
/* $Id: rational.hh,v 1.5 2002/03/05 17:45:30 boenisch Exp $       */
/* Last modified by :                                              */
/* Last modified on :                                              */
/* Update count     :                                              */
/*                                                                 */
/*******************************************************************/
/*                                                                 */
/*  Date      Name               Changes/Extensions                */
/*  ====      ====               ==================                */
/*                                                                 */
/*******************************************************************/


#ifndef _RATIONAL_HH
#define _RATIONAL_HH


/// A class for rational numbers.
template <class T>
class Rational {

private:

  /* ************************************************** */
  /*                     Variables                      */ 
  /* ************************************************** */

  T numerator;
  T denominator;


public:

  /* ************************************************** */
  /*               Methods (public)                     */ 
  /* ************************************************** */

  // Constructors
  Rational();
  Rational(const T& num);
  Rational(const T& num , const T& denom);

  // Access numerator and denominator.
  T get_numerator  () const;
  T get_denominator() const;

  // Arithmetic functions
  Rational<T> operator-();
  Rational<T>& operator+=(const Rational<T>& y);
  Rational<T>& operator-=(const Rational<T>& y);
  Rational<T>& operator*=(const Rational<T>& y);
  Rational<T>& operator/=(const Rational<T>& y);
  Rational<T>& frac();

  // Comparison operators
  bool operator==(const Rational<T>& y) const;
  bool operator!=(const Rational<T>& y) const;
  bool operator< (const Rational<T>& y) const;
  bool operator> (const Rational<T>& y) const;
  bool operator<=(const Rational<T>& y) const;
  bool operator>=(const Rational<T>& y) const;

  /// Divide numerator and denominator by common factors.
  Rational<T>& normalize();

  /// Convert to double.
  double toDouble() const;

}; // class Rational
  

/* ************************************************** */
/*               Non-member functions                 */ 
/* ************************************************** */


template <class T>
Rational<T> operator+(const Rational<T>& x, const Rational<T>& y);

template <class T>
Rational<T> operator-(const Rational<T>& x, const Rational<T>& y);

template <class T>
Rational<T> operator*(const Rational<T>& x, const Rational<T>& y);

template <class T>
Rational<T> operator/(const Rational<T>& x, const Rational<T>& y);

template <class T>
ostream& operator<<(ostream& out, const Rational<T>& x);

template <class T>
istream& operator>>(istream& in, Rational<T>& x);



/* ************************************************** */
/*             Include implementation                 */ 
/* ************************************************** */

#include "rational.cc"


#endif
